"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ClozePattern = void 0;
const utils_1 = require("./utils");
const ClozeRegExp_1 = require("./ClozeRegExp");
const ClozeFieldEnum_1 = require("./ClozeFieldEnum");
const ClozeTypeEnum_1 = require("./ClozeTypeEnum");
const numPatternRegex = new RegExp(`\\[(?:(?:\\\\\\])?[^\\]]?)+?\\d+(?:(?:\\\\\\])?[^\\]]?)+?\\]`);
const hintPatternRegex = new RegExp(`\\[(?:(?:\\\\\\])?[^\\]]?)+?hint(?:(?:\\\\\\])?[^\\]]?)+?\\]`);
const answerKeyword = `answer`; // Must not have regex special characters
class ClozePattern {
    constructor(raw) {
        this.clozeRegexByType = {
            [ClozeTypeEnum_1.ClozeTypeEnum.CLASSIC]: undefined,
            [ClozeTypeEnum_1.ClozeTypeEnum.OVERLAPPING]: undefined,
            [ClozeTypeEnum_1.ClozeTypeEnum.SIMPLE]: undefined,
        };
        this.generateClozeRegexByType = {
            [ClozeTypeEnum_1.ClozeTypeEnum.CLASSIC]: this.generateClozeClassicRegex,
            [ClozeTypeEnum_1.ClozeTypeEnum.OVERLAPPING]: this.generateClozeOLRegex,
            [ClozeTypeEnum_1.ClozeTypeEnum.SIMPLE]: this.generateClozeSimpleRegex,
        };
        this._raw = raw;
        let _numMatch = numPatternRegex.exec(raw);
        let _hintMatch = hintPatternRegex.exec(raw);
        if (!_numMatch) {
            throw new Error("No cloze number pattern found");
        }
        if (!_hintMatch) {
            throw new Error("No cloze hint pattern found");
        }
        if (raw.indexOf(answerKeyword) == -1) {
            throw new Error(`No answer keyword (${answerKeyword}) found in the pattern.`);
        }
        this.numPattern = _numMatch;
        this.hintPattern = _hintMatch;
        this.numRegex = ClozePattern.processPattern(_numMatch[0], (text) => text.replace(/\d+/g, "(\\d+)"));
        this.seqRegex = ClozePattern.processPattern(_numMatch[0], (text) => text.replace(/\d+/g, "([ash]+)"));
        this.hintRegex = ClozePattern.processPattern(_hintMatch[0], (text) => text.replace(/hint/g, "(.+?)"));
        this.hintRegex = "(?:" + this.hintRegex + ")?"; // Cloze hint is always optional
        this._clozeFieldOrder = [ClozeFieldEnum_1.ClozeFieldEnum.answer, ClozeFieldEnum_1.ClozeFieldEnum.hint, ClozeFieldEnum_1.ClozeFieldEnum.seq];
        let positions = {
            [ClozeFieldEnum_1.ClozeFieldEnum.answer]: raw.indexOf(answerKeyword),
            [ClozeFieldEnum_1.ClozeFieldEnum.hint]: this.hintPattern.index,
            [ClozeFieldEnum_1.ClozeFieldEnum.seq]: this.numPattern.index
        };
        // Sort the indexes by their positions
        this._clozeFieldOrder.sort((a, b) => positions[a] - positions[b]);
    }
    static processPattern(text, rplc) {
        let ans = text.substring(1, text.length - 1);
        ans = ans.replace(/\\\[/g, "[").replace(/\\]/g, "]");
        ans = (0, utils_1.escapeRegexString)(ans);
        ans = rplc(ans);
        return ans;
    }
    generateClozeRegexStr(first, firstReplace, second, secondReplace) {
        let begin = this._raw.slice(0, first.index);
        let middle = this._raw.slice(first.index + first[0].length, second.index);
        let ending = this._raw.slice(second.index + second[0].length, this._raw.length);
        let regexStr = (0, utils_1.escapeRegexString)(begin) +
            firstReplace +
            (0, utils_1.escapeRegexString)(middle) +
            secondReplace +
            (0, utils_1.escapeRegexString)(ending);
        regexStr = regexStr.replace(answerKeyword, "(.+?)"); // answerKeyword must not have regex special characters
        return regexStr;
    }
    generateClozeSimpleRegex(pattern) {
        let regexStr;
        if (pattern.numPattern.index < pattern.hintPattern.index) {
            regexStr = pattern.generateClozeRegexStr(pattern.numPattern, "", pattern.hintPattern, pattern.hintRegex);
        }
        else {
            regexStr = pattern.generateClozeRegexStr(pattern.hintPattern, pattern.hintRegex, pattern.numPattern, "");
        }
        let clozeFieldsOrderWithoutSeq = pattern._clozeFieldOrder.filter((x) => x != ClozeFieldEnum_1.ClozeFieldEnum.seq);
        return new ClozeRegExp_1.ClozeRegExp(regexStr, clozeFieldsOrderWithoutSeq, 'g');
    }
    generateClozeClassicRegex(pattern) {
        let regexStr;
        if (pattern.numPattern.index < pattern.hintPattern.index) {
            regexStr = pattern.generateClozeRegexStr(pattern.numPattern, pattern.numRegex, pattern.hintPattern, pattern.hintRegex);
        }
        else {
            regexStr = pattern.generateClozeRegexStr(pattern.hintPattern, pattern.hintRegex, pattern.numPattern, pattern.numRegex);
        }
        return new ClozeRegExp_1.ClozeRegExp(regexStr, pattern._clozeFieldOrder, 'g');
    }
    generateClozeOLRegex(pattern) {
        let regexStr;
        if (pattern.numPattern.index < pattern.hintPattern.index) {
            regexStr = pattern.generateClozeRegexStr(pattern.numPattern, pattern.seqRegex, pattern.hintPattern, pattern.hintRegex);
        }
        else {
            regexStr = pattern.generateClozeRegexStr(pattern.hintPattern, pattern.hintRegex, pattern.numPattern, pattern.seqRegex);
        }
        return new ClozeRegExp_1.ClozeRegExp(regexStr, pattern._clozeFieldOrder, 'g');
    }
    get clozeFieldsOrder() {
        return this._clozeFieldOrder;
    }
    getClozeRegex(clozeType) {
        let clozeRegex = this.clozeRegexByType[clozeType];
        if (clozeRegex != undefined) {
            clozeRegex.regex.lastIndex = 0; // Reset the regex
            return clozeRegex;
        }
        clozeRegex = this.generateClozeRegexByType[clozeType](this);
        this.clozeRegexByType[clozeType] = clozeRegex;
        return clozeRegex;
    }
    hasClozeType(text, clozeType) {
        for (const priorityType of ClozeTypeEnum_1.ClozeTypesPriority) {
            if (this.getClozeRegex(priorityType).test(text)) {
                return clozeType == priorityType;
            }
        }
        return false;
    }
    getClozeTypes(text) {
        const clozeTypes = [];
        for (const priorityType of ClozeTypeEnum_1.ClozeTypesPriority) {
            if (this.getClozeRegex(priorityType).test(text)) {
                clozeTypes.push(priorityType);
            }
        }
        return clozeTypes;
    }
    getMainClozeType(text) {
        for (const priorityType of ClozeTypeEnum_1.ClozeTypesPriority) {
            if (this.getClozeRegex(priorityType).test(text)) {
                return priorityType;
            }
        }
        return null;
    }
}
exports.ClozePattern = ClozePattern;
